package mixins

import (
    "context"
    "entgo.io/ent"
    "entgo.io/ent/dialect"
    "entgo.io/ent/dialect/sql"
    "entgo.io/ent/schema/field"
    "entgo.io/ent/schema/mixin"
)

// SoftDeleteMixin implements the ent.Mixin for sharing
// soft_delete field with package schemas.
// see also: source code for more information
type SoftDeleteMixin struct {
    // We embed the `mixin.Schema` to avoid
    // implementing the rest of the methods.
    mixin.Schema
}

func (SoftDeleteMixin) Fields() []ent.Field {
    return []ent.Field{
        field.Time("deleted_at").
            Optional().
            SchemaType(map[string]string{
                dialect.MySQL: "datetime",
            }).
            Comment("Soft delete | 软删除（逻辑删除）"),
    }
}

type SoftDeleteKey struct{}

// SkipSoftDelete returns a new context that skips the soft-delete interceptor/mutators.
func SkipSoftDelete(parent context.Context) context.Context {
    return context.WithValue(parent, SoftDeleteKey{}, true)
}

//// Interceptors of the SoftDeleteMixin.
//func (d SoftDeleteMixin) Interceptors() []ent.Interceptor {
//    return []ent.Interceptor{
//        intercept.TraverseFunc(func(ctx context.Context, q intercept.Query) error {
//            // Skip soft-delete, means include soft-deleted entities.
//            if skip, _ := ctx.Value(SoftDeleteKey{}).(bool); skip {
//                return nil
//            }
//            d.SoftDeleteWhere(q)
//            return nil
//        }),
//    }
//}

//// Hooks of the SoftDeleteMixin.
//func (d SoftDeleteMixin) Hooks() []ent.Hook {
//    return []ent.Hook{
//        hook.On(
//            func(next ent.Mutator) ent.Mutator {
//                return ent.MutateFunc(func(ctx context.Context, m ent.Mutation) (ent.Value, error) {
//                    // Skip soft-delete, means delete the entity permanently.
//                    if skip, _ := ctx.Value(SoftDeleteKey{}).(bool); skip {
//                        return next.Mutate(ctx, m)
//                    }
//                    mx, ok := m.(interface {
//                        SetOp(ent.Op)
//                        SetDeletedAt(time.Time)
//                        WhereP(...func(*sql.Selector))
//                    })
//                    if !ok {
//                        return nil, fmt.Errorf("unexpected mutation type %T", m)
//                    }
//                    d.SoftDeleteWhere(mx)
//                    mx.SetOp(ent.OpUpdate)
//                    mx.SetDeletedAt(time.Now())
//
//                    return next.Mutate(ctx, m)
//                    //return mx.Client().Mutate(ctx, m)
//                })
//            },
//            ent.OpDeleteOne|ent.OpDelete,
//        ),
//    }
//}

// SoftDeleteWhere adds a storage-level predicate to the queries and mutations.
func (d SoftDeleteMixin) SoftDeleteWhere(w interface{ WhereP(...func(*sql.Selector)) }) {
    w.WhereP(
        sql.FieldIsNull(d.Fields()[0].Descriptor().Name),
    )
}
